class Translated:
    def __init__(self, original=None, translated=None, pronounce_original=None, pronounce_translated=None):
        self.original = original
        self.translated = translated
        self.pronounce_original = pronounce_original
        self.pronounce_translated = pronounce_translated
